﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace caRoads
{
    class Map
    {
        List<Town> Towns;
        List<RoadType> RoadTypes;
        List<Condition> Conditions;
        List<Region> Regions;

        double money;

        List<Road> AvailableRoads;
        List<Road> BuiltRoads;

        double spentMoney
        {
            get
            {
                return BuiltRoads.Sum(br => br.Price);
            }
        }

        public Map()
        {
            Towns = new List<Town>();
            RoadTypes = new List<RoadType>();
            Conditions = new List<Condition>();
            Regions = new List<Region>();

            AvailableRoads = new List<Road>();
            BuiltRoads = new List<Road>();

            money = 0;
        }

        public bool IsValid
        {
            get
            {
                return Towns.Count >= 2
                    && RoadTypes.Count >= 1
                    && money > 0;
            }
        }

        public void Clear()
        {
            Towns.Clear();
            RoadTypes.Clear();
            Conditions.Clear();
            Regions.Clear();
        }


        public bool Load(string fileName)
        {
            Clear();

            try
            {
                string[] lines = File.ReadAllLines(fileName);
                string[] headers = new string[] { "MESTA", "SILNICE", "FINANCE", "PODMINKY", "KRAJINA" };
                string currentHeader = string.Empty;
                Thread.CurrentThread.CurrentCulture = new CultureInfo("en-US");
                for (int i = 0; i < lines.Length; i++)
                {
                    string line = lines[i].Trim();
                    if (string.IsNullOrEmpty(line))
                        continue;
                    if (headers.Contains(line))
                    {
                        currentHeader = line;
                        continue;
                    }
                    switch (currentHeader)
                    {
                        case "MESTA":
                            Towns.Add(Town.Parse(line));
                            break;
                        case "SILNICE":
                            RoadTypes.Add(RoadType.Parse(line));
                            break;
                        case "FINANCE":
                            money = double.Parse(line);
                            break;
                        case "PODMINKY":
                            Conditions.Add(Condition.Parse(line, Towns));
                            break;
                        case "KRAJINA":
                            Region region = Region.Parse(line);
                            if (region.Coefficient != 1)
                                Regions.Add(region);
                            break;
                        default: break;
                    }
                }
                RoadTypes = RoadTypes.OrderBy(rt => rt.Price).ToList(); // ToDo: create comparer for Sort method
                return IsValid;
            }
            catch 
            {
                return false;
            }
        }

        public bool FindSolution()
        {
            GenerateAvailableRoads();
            AvailableRoads = AvailableRoads.OrderBy(ar => ar.Duration).ToList();
            return false;
        }

        public void GenerateAvailableRoads()
        {
            AvailableRoads.Clear();
            for (int i = 0; i < Towns.Count -1; i++)
            {
                for (int ii = i + 1; ii < Towns.Count; ii++)
                {
                    AvailableRoads.Add(new Road(Towns[i], Towns[ii], RoadTypes[0]));
                }  
            }
        }

        public void AplyRegions()
        {
            foreach (Region region in this.Regions)
            {
                for (int iRoad = 0; iRoad < AvailableRoads.Count; iRoad++)
                {
                    Road currentRoad = AvailableRoads[iRoad];
                    for (int iRoadPard = 0; iRoadPard < currentRoad.RoadParts.Count(); iRoadPard++)
                    {
                        RoadPart currentRoadPart = currentRoad.RoadParts.ElementAt(iRoadPard);
                        List<Location> intersections = region.GetIntersectionWithRoadPart(currentRoadPart);
                        if (intersections.Count == 2)
                        {
                            currentRoad.SplitRoadPart(currentRoadPart, intersections, region.Coefficient);
                            break;
                        }
                    }
                }
            }
        }

        public void CreateMinimalSkeleton(SkeletonType type)
        {
            IEnumerable<Road> roads = null;
            switch (type)
            {
                case SkeletonType.Duration:
                    roads = AvailableRoads.OrderBy(r => r.Duration);
                    break;
                case SkeletonType.Price:
                    roads = AvailableRoads.OrderBy(r => r.Price);
                    break;
                default:
                    throw new NotImplementedException();
            }
            BuiltRoads.Clear();
            int position = 0;
            while (BuiltRoads.Count < Towns.Count - 1)
            {
                Road road = roads.ElementAt(position);
                if (!Way.ExistWay(road.TownFrom, road.TownTo, BuiltRoads))
                    BuiltRoads.Add(road);
                position++;
            }
        }


        internal void ShowBuiltRoads()
        {
            Console.WriteLine("Available money: {0:F2}", money); 
            foreach (Road road in BuiltRoads)
            {
                Console.WriteLine(road);
            }
            Console.WriteLine("Spent money: {0:F2}", spentMoney);
        }

        internal void CheckConstraints()
        {
            bool satisfied = true;
            foreach (Condition condition in Conditions)
                satisfied = satisfied & condition.IsSatisfied(BuiltRoads);
            if (satisfied)
                Console.WriteLine("Nalezena silnicni sit splnuje pozadavky. :-)");
            else
                Console.WriteLine("Nalezena silnicni sit nesplnuje pozadavky. :-(");
        }
    }
}
